// [TPT] - IP Country

#include "stdafx.h"
#include "emule.h"
#include "IP2Country.h"
#include "otherfunctions.h"
#include "Preferences.h"
#include <flag/resource.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	NO_FLAG	65535

CIP2Country::CIP2Country(){
	gi = NULL;
	LoadFromFile();
	// Load dll flag	
	_hCountryFlagDll = LoadLibrary(theApp.glob_prefs->GetConfigDir()+"countryflag.dll"); 
	if (_hCountryFlagDll == NULL) 	
		AddLogLine(false,_T("Failed to load countryflag.dll"));
	else		
		AddLogLine(false, "Country Flags have been loaded");			
}

CIP2Country::~CIP2Country(){
	if (gi != NULL)
		GeoIP_delete(gi);
	country_flags.RemoveAll();
	// Release dll flag
	if(::FreeLibrary(_hCountryFlagDll) == TRUE)
		_hCountryFlagDll = NULL;
}

void CIP2Country::LoadFromFile(){	
	const char* configDir = theApp.glob_prefs->GetConfigDir();	

	// load CITY IP database	
	gi = GeoIP_open_type(GEOIP_CITY_EDITION_REV1, GEOIP_STANDARD, configDir);
	if (gi != NULL) 
	{
		AddLogLine(false, "GeoIp City Loaded");
		AddLogLine(false, "GeoIp City is provided by Maxmind (http://www.maxmind.com) with charge");
		db_loaded = GEOIP_CITY_EDITION_REV1;
		return;
	}

	// load REGION IP database	
	gi = GeoIP_open_type(GEOIP_REGION_EDITION_REV1, GEOIP_STANDARD, configDir);
	if (gi != NULL) 
	{
		AddLogLine(false, "GeoIp Region Loaded");
		AddLogLine(false, "GeoIp Region is provided by Maxmind, http://www.maxmind.com, with charge");
		db_loaded = GEOIP_REGION_EDITION_REV1;
		return;
	}

	// load COUNTRY IP database	
	gi = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD, configDir);
	if (gi != NULL) 
	{
		AddLogLine(false, "GeoIp Country Loaded");
		AddLogLine(false, "GeoIp Country is provided by Maxmind, http://www.maxmind.com, free of charge");
		AddLogLine(false, "You can download latest version in here: http://www.maxmind.com/download/geoip/database/");
		db_loaded = GEOIP_COUNTRY_EDITION;
		return;
	}
	
	AddLogLine(false, "GeoIp Country not loaded, couldn't find any GeoIP file in config dir");
	AddLogLine(false, "GeoIp Country is provided by Maxmind, http://www.maxmind.com, free of charge");
	AddLogLine(false, "You can download latest version in here: http://www.maxmind.com/download/geoip/database/");
	return;
}


CString CIP2Country::GetCountryFromIP(LPCSTR fullUserIP){	
	if (gi == NULL)
		return "";

	if (db_loaded == GEOIP_CITY_EDITION_REV1)
	{
		GeoIPRecord * gir;
		gir = GeoIP_record_by_addr (gi, fullUserIP);
		CString City;
		if (gir != NULL) {		
			City.Format("%s [%s]", gir->city, gir->country_name);			
			GeoIPRecord_delete(gir);
		}		
		return City;
	}

	if (db_loaded == GEOIP_REGION_EDITION_REV1)
	{
		GeoIPRegion * gir;
		gir = GeoIP_region_by_name(gi, fullUserIP);
		CString region;
		if (gir != NULL) {			
			region.Format("%s, %s",
				 gir->country_code,
				 (!gir->region[0]) ? "N/A" : gir->region );
			GeoIPRegion_delete(gir);
		} 		
		return region;
	}
	
	return GeoIP_country_name_by_addr(gi, fullUserIP);


}

CString CIP2Country::GetCountryISOFromIP(LPCSTR fullUserIP){	
	if (gi == NULL)
		return "";

	if (db_loaded == GEOIP_CITY_EDITION_REV1)
	{
		GeoIPRecord * gir;
		gir = GeoIP_record_by_addr (gi, fullUserIP);
		CString City;
		if (gir != NULL) {			
			City.Format("%s", gir->country_code);			
			GeoIPRecord_delete(gir);
		}		
		return City;
	}

	if (db_loaded == GEOIP_REGION_EDITION_REV1)
	{
		GeoIPRegion * gir;
		gir = GeoIP_region_by_name(gi, fullUserIP);
		CString region;
		if (gir != NULL) {			
			region.Format("%s", gir->country_code);
			GeoIPRegion_delete(gir);
		} 		
		return region;
	}

	return GeoIP_country_code_by_addr(gi, fullUserIP);
}

int CIP2Country::Compare(CString country1, CString country2)
{
	if (db_loaded == GEOIP_CITY_EDITION_REV1)
	{
		// Compare countries
		int find1 = country1.Find('[');
		int find2 = country2.Find('[');
		if ((find1 != -1) && (find2 != -1))
		{
			int compare = _tcsicmp(country1.Mid( find1 ), country2.Mid( country2.Find('[') ));
			if ( compare == 0 )
				return _tcsicmp(country1, country2);
			else
				return compare;
		}
		else
			if (find1 != -1)
				return 1;
			else 
				return -1;
	}

	return _tcsicmp(country1, country2);
}

CxImage CIP2Country::GetCountryFlag(LPCSTR fullUserIP)
{	
	CxImage flag;	
	if (_hCountryFlagDll == NULL)
	{
		flag.Enable(false);
		return flag;
	}

	CString countryISO = GetCountryISOFromIP(fullUserIP);
	
	if (country_flags.Lookup(countryISO, flag))
	{
		flag.Enable(true);
		return flag;
	}
	else
	{
			uint16	resID[] = {
				IDI_COUNTRY_FLAG_AD, IDI_COUNTRY_FLAG_AE, IDI_COUNTRY_FLAG_AF, IDI_COUNTRY_FLAG_AG, 
				IDI_COUNTRY_FLAG_AI, IDI_COUNTRY_FLAG_AL, IDI_COUNTRY_FLAG_AM, IDI_COUNTRY_FLAG_AN, 
				IDI_COUNTRY_FLAG_AO, IDI_COUNTRY_FLAG_AR, IDI_COUNTRY_FLAG_AS, IDI_COUNTRY_FLAG_AT, 
				IDI_COUNTRY_FLAG_AU, IDI_COUNTRY_FLAG_AW, IDI_COUNTRY_FLAG_AZ, IDI_COUNTRY_FLAG_BA, 
				IDI_COUNTRY_FLAG_BB, IDI_COUNTRY_FLAG_BD, IDI_COUNTRY_FLAG_BE, IDI_COUNTRY_FLAG_BF, 
				IDI_COUNTRY_FLAG_BG, IDI_COUNTRY_FLAG_BH, IDI_COUNTRY_FLAG_BI, IDI_COUNTRY_FLAG_BJ, 
				IDI_COUNTRY_FLAG_BM, IDI_COUNTRY_FLAG_BN, IDI_COUNTRY_FLAG_BO, IDI_COUNTRY_FLAG_BR, 
				IDI_COUNTRY_FLAG_BS, IDI_COUNTRY_FLAG_BT, IDI_COUNTRY_FLAG_BW, IDI_COUNTRY_FLAG_BY, 
				IDI_COUNTRY_FLAG_BZ, IDI_COUNTRY_FLAG_CA, IDI_COUNTRY_FLAG_CC, IDI_COUNTRY_FLAG_CD, 
				IDI_COUNTRY_FLAG_CF, IDI_COUNTRY_FLAG_CG, IDI_COUNTRY_FLAG_CH, IDI_COUNTRY_FLAG_CI, 
				IDI_COUNTRY_FLAG_CK, IDI_COUNTRY_FLAG_CL, IDI_COUNTRY_FLAG_CM, IDI_COUNTRY_FLAG_CN, 
				IDI_COUNTRY_FLAG_CO, IDI_COUNTRY_FLAG_CR, IDI_COUNTRY_FLAG_CU, IDI_COUNTRY_FLAG_CV, 
				IDI_COUNTRY_FLAG_CX, IDI_COUNTRY_FLAG_CY, IDI_COUNTRY_FLAG_CZ, IDI_COUNTRY_FLAG_DE, 
				IDI_COUNTRY_FLAG_DJ, IDI_COUNTRY_FLAG_DK, IDI_COUNTRY_FLAG_DM, IDI_COUNTRY_FLAG_DO, 
				IDI_COUNTRY_FLAG_DZ, IDI_COUNTRY_FLAG_EC, IDI_COUNTRY_FLAG_EE, IDI_COUNTRY_FLAG_EG, 
				IDI_COUNTRY_FLAG_EH, IDI_COUNTRY_FLAG_ER, IDI_COUNTRY_FLAG_ES, IDI_COUNTRY_FLAG_ET, 
				IDI_COUNTRY_FLAG_FI, IDI_COUNTRY_FLAG_FJ, IDI_COUNTRY_FLAG_FK, IDI_COUNTRY_FLAG_FM, 
				IDI_COUNTRY_FLAG_FO, IDI_COUNTRY_FLAG_FR, IDI_COUNTRY_FLAG_GA, IDI_COUNTRY_FLAG_GB, 
				IDI_COUNTRY_FLAG_GD, IDI_COUNTRY_FLAG_GE, IDI_COUNTRY_FLAG_GG, IDI_COUNTRY_FLAG_GH, 
				IDI_COUNTRY_FLAG_GI, IDI_COUNTRY_FLAG_GK, IDI_COUNTRY_FLAG_GL, IDI_COUNTRY_FLAG_GM, 
				IDI_COUNTRY_FLAG_GN, IDI_COUNTRY_FLAG_GP, IDI_COUNTRY_FLAG_GQ, IDI_COUNTRY_FLAG_GR, 
				IDI_COUNTRY_FLAG_GS, IDI_COUNTRY_FLAG_GT, IDI_COUNTRY_FLAG_GU, IDI_COUNTRY_FLAG_GW, 
				IDI_COUNTRY_FLAG_GY, IDI_COUNTRY_FLAG_HK, IDI_COUNTRY_FLAG_HN, IDI_COUNTRY_FLAG_HR, 
				IDI_COUNTRY_FLAG_HT, IDI_COUNTRY_FLAG_HU, IDI_COUNTRY_FLAG_ID, IDI_COUNTRY_FLAG_IE, 
				IDI_COUNTRY_FLAG_IL, IDI_COUNTRY_FLAG_IM, IDI_COUNTRY_FLAG_IN, IDI_COUNTRY_FLAG_IO, 
				IDI_COUNTRY_FLAG_IQ, IDI_COUNTRY_FLAG_IR, IDI_COUNTRY_FLAG_IS, IDI_COUNTRY_FLAG_IT, 
				IDI_COUNTRY_FLAG_JE, IDI_COUNTRY_FLAG_JM, IDI_COUNTRY_FLAG_JO, IDI_COUNTRY_FLAG_JP, 
				IDI_COUNTRY_FLAG_KE, IDI_COUNTRY_FLAG_KG, IDI_COUNTRY_FLAG_KH, IDI_COUNTRY_FLAG_KI, 
				IDI_COUNTRY_FLAG_KM, IDI_COUNTRY_FLAG_KN, IDI_COUNTRY_FLAG_KP, IDI_COUNTRY_FLAG_KR, 
				IDI_COUNTRY_FLAG_KW, IDI_COUNTRY_FLAG_KY, IDI_COUNTRY_FLAG_KZ, IDI_COUNTRY_FLAG_LA, 
				IDI_COUNTRY_FLAG_LB, IDI_COUNTRY_FLAG_LC, IDI_COUNTRY_FLAG_LI, IDI_COUNTRY_FLAG_LK, 
				IDI_COUNTRY_FLAG_LR, IDI_COUNTRY_FLAG_LS, IDI_COUNTRY_FLAG_LT, IDI_COUNTRY_FLAG_LU, 
				IDI_COUNTRY_FLAG_LV, IDI_COUNTRY_FLAG_LY, IDI_COUNTRY_FLAG_MA, IDI_COUNTRY_FLAG_MC, 
				IDI_COUNTRY_FLAG_MD, IDI_COUNTRY_FLAG_MG, IDI_COUNTRY_FLAG_MH, IDI_COUNTRY_FLAG_MK, 
				IDI_COUNTRY_FLAG_ML, IDI_COUNTRY_FLAG_MM, IDI_COUNTRY_FLAG_MN, IDI_COUNTRY_FLAG_MO, 
				IDI_COUNTRY_FLAG_MP, IDI_COUNTRY_FLAG_MQ, IDI_COUNTRY_FLAG_MR, IDI_COUNTRY_FLAG_MS, 
				IDI_COUNTRY_FLAG_MT, IDI_COUNTRY_FLAG_MU, IDI_COUNTRY_FLAG_MV, IDI_COUNTRY_FLAG_MW, 
				IDI_COUNTRY_FLAG_MX, IDI_COUNTRY_FLAG_MY, IDI_COUNTRY_FLAG_MZ, IDI_COUNTRY_FLAG_NA, 
				IDI_COUNTRY_FLAG_NC, IDI_COUNTRY_FLAG_NE, IDI_COUNTRY_FLAG_NF, IDI_COUNTRY_FLAG_NG, 
				IDI_COUNTRY_FLAG_NI, IDI_COUNTRY_FLAG_NL, IDI_COUNTRY_FLAG_NO, IDI_COUNTRY_FLAG_NP, 
				IDI_COUNTRY_FLAG_NR, IDI_COUNTRY_FLAG_NU, IDI_COUNTRY_FLAG_NZ, IDI_COUNTRY_FLAG_OM, 
				IDI_COUNTRY_FLAG_PA, IDI_COUNTRY_FLAG_PC, IDI_COUNTRY_FLAG_PE, IDI_COUNTRY_FLAG_PF, 
				IDI_COUNTRY_FLAG_PG, IDI_COUNTRY_FLAG_PH, IDI_COUNTRY_FLAG_PK, IDI_COUNTRY_FLAG_PL, 
				IDI_COUNTRY_FLAG_PM, IDI_COUNTRY_FLAG_PN, IDI_COUNTRY_FLAG_PR, IDI_COUNTRY_FLAG_PS, 
				IDI_COUNTRY_FLAG_PT, IDI_COUNTRY_FLAG_PW, IDI_COUNTRY_FLAG_PY, IDI_COUNTRY_FLAG_QA, 
				IDI_COUNTRY_FLAG_RO, IDI_COUNTRY_FLAG_RU, IDI_COUNTRY_FLAG_RW, IDI_COUNTRY_FLAG_SA, 
				IDI_COUNTRY_FLAG_SB, IDI_COUNTRY_FLAG_SC, IDI_COUNTRY_FLAG_SD, IDI_COUNTRY_FLAG_SE, 
				IDI_COUNTRY_FLAG_SG, IDI_COUNTRY_FLAG_SH, IDI_COUNTRY_FLAG_SI, IDI_COUNTRY_FLAG_SK, 
				IDI_COUNTRY_FLAG_SL, IDI_COUNTRY_FLAG_SM, IDI_COUNTRY_FLAG_SN, IDI_COUNTRY_FLAG_SO, 
				IDI_COUNTRY_FLAG_SR, IDI_COUNTRY_FLAG_ST, IDI_COUNTRY_FLAG_SU, IDI_COUNTRY_FLAG_SV, 
				IDI_COUNTRY_FLAG_SY, IDI_COUNTRY_FLAG_SZ, IDI_COUNTRY_FLAG_TC, IDI_COUNTRY_FLAG_TD, 
				IDI_COUNTRY_FLAG_TF, IDI_COUNTRY_FLAG_TG, IDI_COUNTRY_FLAG_TH, IDI_COUNTRY_FLAG_TJ, 
				IDI_COUNTRY_FLAG_TK, IDI_COUNTRY_FLAG_TL, IDI_COUNTRY_FLAG_TM, IDI_COUNTRY_FLAG_TN, 
				IDI_COUNTRY_FLAG_TO, IDI_COUNTRY_FLAG_TR, IDI_COUNTRY_FLAG_TT, IDI_COUNTRY_FLAG_TV, 
				IDI_COUNTRY_FLAG_TW, IDI_COUNTRY_FLAG_TZ, IDI_COUNTRY_FLAG_UA, IDI_COUNTRY_FLAG_UG, 
				IDI_COUNTRY_FLAG_UM, IDI_COUNTRY_FLAG_US, IDI_COUNTRY_FLAG_UY, IDI_COUNTRY_FLAG_UZ, 
				IDI_COUNTRY_FLAG_VA, IDI_COUNTRY_FLAG_VC, IDI_COUNTRY_FLAG_VE, IDI_COUNTRY_FLAG_VG, 
				IDI_COUNTRY_FLAG_VI, IDI_COUNTRY_FLAG_VN, IDI_COUNTRY_FLAG_VU, IDI_COUNTRY_FLAG_WF, 
				IDI_COUNTRY_FLAG_WS, IDI_COUNTRY_FLAG_YE, IDI_COUNTRY_FLAG_YU, IDI_COUNTRY_FLAG_ZA, 
				IDI_COUNTRY_FLAG_ZM, IDI_COUNTRY_FLAG_ZW, 
				IDI_COUNTRY_FLAG_UK,

				//http://home.12move.nl/~sh829487/ukvlaggen.gif
				//http://geography.about.com/library/faq/blqzuk.htm, by tharghan

				NO_FLAG};

			CString countryID[] = {
				"AD", "AE", "AF", "AG", "AI", "AL", "AM", "AN", "AO", "AR", "AS", "AT", "AU", "AW", "AZ", 
				"BA", "BB", "BD", "BE", "BF", "BG", "BH", "BI", "BJ", "BM", "BN", "BO", "BR", "BS", "BT", 
				"BW", "BY", "BZ", "CA", "CC", "CD", "CF", "CG", "CH", "CI", "CK", "CL", "CM", "CN", "CO", 
				"CR", "CU", "CV", "CX", "CY", "CZ", "DE", "DJ", "DK", "DM", "DO", "DZ", "EC", "EE", "EG", 
				"EH", "ER", "ES", "ET", "FI", "FJ", "FK", "FM", "FO", "FR", "GA", "GB", "GD", "GE", "GG", 
				"GH", "GI", "GK", "GL", "GM", "GN", "GP", "GQ", "GR", "GS", "GT", "GU", "GW", "GY", "HK", 
				"HN", "HR", "HT", "HU", "ID", "IE", "IL", "IM", "IN", "IO", "IQ", "IR", "IS", "IT", "JE", 
				"JM", "JO", "JP", "KE", "KG", "KH", "KI", "KM", "KN", "KP", "KR", "KW", "KY", "KZ", "LA", 
				"LB", "LC", "LI", "LK", "LR", "LS", "LT", "LU", "LV", "LY", "MA", "MC", "MD", "MG", "MH", 
				"MK", "ML", "MM", "MN", "MO", "MP", "MQ", "MR", "MS", "MT", "MU", "MV", "MW", "MX", "MY", 
				"MZ", "NA", "NC", "NE", "NF", "NG", "NI", "NL", "NO", "NP", "NR", "NU", "NZ", "OM", "PA", 
				"PC", "PE", "PF", "PG", "PH", "PK", "PL", "PM", "PN", "PR", "PS", "PT", "PW", "PY", "QA", 
				"RO", "RU", "RW", "SA", "SB", "SC", "SD", "SE", "SG", "SH", "SI", "SK", "SL", "SM", "SN", 
				"SO", "SR", "ST", "SU", "SV", "SY", "SZ", "TC", "TD", "TF", "TG", "TH", "TJ", "TK", "TL", 
				"TM", "TN", "TO", "TR", "TT", "TV", "TW", "TZ", "UA", "UG", "UM", "US", "UY", "UZ", "VA", 
				"VC", "VE", "VG", "VI", "VN", "VU", "WF", "WS", "YE", "YU", "ZA", "ZM", "ZW", 
				"UK",
				//http://home.12move.nl/~sh829487/ukvlaggen.gif
				//http://geography.about.com/library/faq/blqzuk.htm, by tharghan

				"N/A"
			};

			for(int cur_pos = 0; resID[cur_pos] != NO_FLAG; cur_pos++)
			{
				if (countryISO.CompareNoCase(countryID[cur_pos]) == 0)
				{				            
					if (flag.CreateFromHICON(LoadIcon(_hCountryFlagDll, MAKEINTRESOURCE(resID[cur_pos]))))
					{
						flag.Enable(true);
						country_flags.SetAt(countryISO, flag);				
					}
					else
						flag.Enable(false);
                    
					return flag;
				}
			}
		}
	
	flag.Enable(false);
	return flag;
}